<?php
class DB_Reg
{
	/**
	 * @var mixed
	 */
	protected $_id = null;
	/**
	 * @var string
	 */
	protected $_tabla = '';
	/**
	 * @var string
	 */
	protected $_campoid = 'id';
	/**
	 * @var array
	 */
	protected $_data = array();
	/**
	 * @var array
	 */
	protected $_fields = array();
	/**
	 * @var array
	 */
	protected $_camposreadonly = array();
	/**
	 * @var array
	 */
	protected $_camposexcluidos = array();
	/**
	 * @var array
	 */
	protected $_camposencriptados = array();
	/**
	 * @var string
	 */
	protected $_funcionencriptar = 'md5';
    /**
	 * @var array
	 */
	protected $_camposserializados = array();
	/**
	 * @var DB_Connection
	 */
	protected $_db;
	/**
	 * @var array
	 */
	protected $_errores;

	/**
	 * Constructor
	 *
	 * @param DB_Connection $db
	 * @param string $tabla
	 * @param string $campoid
	 */
	public function __construct(&$db, $tabla, $campoid="id")
	{
		$this->_db = &$db;
		$this->_tabla = $tabla;
		$this->_campoid = $campoid;

		$this->_fields = $this->_getFields();

		$this->ResetData();
	}
	/**
	 * Asigna el identificador de la conexión a la base de datos
	 *
     * @param resource
     */
    public function SetLinkIdentifier($dbcon)
    {
    	$this->_db->SetLinkIdentifier($dbcon);
    }
    /**
     * Obtiene los nombres de los campos de la tabla
     *
     * @return array
     */
    protected function _getFields()
    {
		return $this->_db->GetRows(sprintf("SHOW FIELDS FROM `%s`", $this->_tabla), true, false, 'Field', true);
    }
	/**
	 * Resetea los datos (campos)
	 */
	public function ResetData()
	{
		$this->_data = array();

		foreach($this->_fields as $field)
		{
			if($field['Field'] == $this->_campoid) //if($field["Key"] == "PRI")
			{
				//$this->_id = $this->_type2val($field["Type"]); //id no lo reseteamos
			}
			elseif(!in_array($field['Field'], $this->_camposexcluidos))
			{
				$this->_data[$field['Field']] = $field['Default'] != '' ? $field['Default'] : $this->_type2val($field['Type']);
			}
		}

		$this->_errores = array();
	}
	/**
	 * Carga un registro
	 *
	 * @param mixed $id
	 * @return bool
	 */
	public function Load($id=null)
	{
		if(!$this->_idVacio($id))
		{
			$this->_id = $id;
		}

		if($this->Exists())
		{
			$this->_data = $this->_db->GetRow(sprintf("SELECT * FROM `%s` WHERE %s = %s", $this->_tabla, $this->_campoid, $this->_db->quote($this->_id)), true, false);
            unset($this->_data[$this->_campoid]);
			$this->UnserializarCamposSerializados();
			return true;
		}

		$this->_errores[] = 'No existe el registro '.$this->_campoid.' = '.$this->_id;
		$this->ResetData();

		return false;
	}

	public function LoadByFields($values, $fieldnames)
	{
		$size_valores = sizeof($values);
		$size_campos = sizeof($fieldnames);

		if($size_valores > 0 && $size_valores == $size_campos)
		{
			if($this->ExistsByFields($values, $fieldnames))
			{
				$query = "SELECT * FROM `".$this->_tabla."` WHERE 1";

				for($c = 0; $c < $size_valores; $c++)
				{
					$query .= sprintf(" AND `%s` = '%s'", $fieldnames[$c], addcslashes($values[$c], "\\'"));
				}

				$this->_data = $this->_db->GetRow($query, true, false);
				$this->_id = $this->_data[$this->_campoid];
				unset($this->_data[$this->_campoid]);
                $this->UnserializarCamposSerializados();
				return true;
			}
			else
			{
				$this->_errores[] = 'No existe el registro';
				$this->ResetData();
			}
		}
		else
		{
			$this->_errores[] = 'Array de values y/o de fieldnames no válido';
		}

		return false;
	}

    protected function UnserializarCamposSerializados()
    {
        foreach($this->_camposserializados as $campo)
        {
            if(!empty($this->_data[$campo]))
            {
                $this->_data[$campo] = unserialize($this->_data[$campo]);
            }
        }
    }

    protected function SerializarCamposSerializados(&$data)
    {
        foreach($this->_camposserializados as $campo)
        {
            if(!empty($data[$campo]))
            {
                $data[$campo] = serialize($data[$campo]);
            }
        }
    }
	/**
	 * De los elementos de $_data devuelve sólo los que figuran en la tabla como campos
	 *
	 * @return array
	 */
	public function GetDataFields()
	{
		$data = array();
		foreach($this->_data as $key=>$value)
		{
			if(array_key_exists($key, $this->_fields) && $key != $this->_campoid)
			{
				$data[$key] = $value;
			}
		}

		return $data;
	}
	/**
	 * Guarda el registro
	 *
	 * @return bool
	 */
	public function Save()
	{
		$data = $this->GetDataFields();
        $this->SerializarCamposSerializados($data);

		if($this->_idVacio($this->_id))
		{
			if($this->_db->Execute(buildQuery_Insert($data, $this->_tabla)))
			{
				$this->_id = $this->_db->Insert_id();
				return true;
			}
			else
			{
				$this->_errores[] = $this->_db->Error();
			}
		}
		elseif($this->Exists())
		{
			if($this->_db->Execute(buildQuery_Update($data, $this->_tabla, array($this->_campoid=>$this->_id))))
			{
				return true;
			}
			else
			{
				$this->_errores[] = $this->_db->Error();
			}
		}
		else
		{
			//Tengo un id pero no existe el registro correspondiente, creo uno con el mismo id
			$data[$this->_campoid] = $this->_id;
			if($this->_db->Execute(buildQuery_Insert($data, $this->_tabla)))
			{
				return true;
			}
			else
			{
				$this->_errores[] = $this->_db->Error();
			}
		}

		return false;
	}
	/**
	 * Establece los valores de los campos
	 *
	 * @param array $data
	 * @return bool
	 */
	public function SetData($data)
	{
		$ok = true;

		foreach($data as $key=>$value)
		{
			if($key == $this->_campoid)
			{
				$this->_id = $value;
			}
			else
			{
				$ok &= $this->SetValue($key, $value);
			}
		}

		return $ok;
	}
	/**
	 * Establece el valor de un campo
	 *
	 * @param string $campo
	 * @param mixed $valor
	 * @return bool
	 */
	public function SetValue($campo, $valor)
	{
		if(isset($this->_data[$campo]))
		{
			if(!in_array($campo, $this->_camposreadonly))
			{
				$this->_data[$campo] = in_array($campo, $this->_camposencriptados) ? eval('return '.$this->_funcionencriptar.'($valor);') : $valor;

				return true;
			}
			else
			{
				$this->_errores[] = 'El campo '.$campo.' es de sólo lectura';
			}
		}
		else
		{
			$this->_errores[] = 'No existe el campo '.$campo;
		}

		return false;
	}
	/**
	 * Establece el valor de un campo, aunque sea readonly
	 *
	 * @param string $campo
	 * @param mixed $valor
	 * @return bool
	 */
	protected function _setValue($campo, $valor)
	{
		if(isset($this->_data[$campo]))
		{
			$this->_data[$campo] = in_array($valor, $this->_camposencriptados) ? eval('return '.$this->_funcionencriptar.'($valor);') : $valor;

			return true;
		}
		else
		{
			$this->_errores[] = 'No existe el campo '.$campo;
		}

		return false;
	}
	/**
	 * Obtiene el array asociativo con los valores de los campos actualmente cargados
	 *
	 * @param bool $htmlentities
	 * @return array
	 */
	public function GetData($htmlentities=true)
	{
        $arr = $this->_data;

		if($htmlentities)
		{
			foreach($arr as $key=>$val)
			{
                if(!in_array($key, $this->_camposserializados))
                {
                    $arr[$key] = $this->HtmlEntities($val);
                }
			}
		}

		$arr[$this->_campoid] = $htmlentities ? $this->HtmlEntities($this->_id) : $this->_id;

		return $arr;
	}
	/**
	 * Obtiene el valor de un campo (null si no lo encuentra)
	 *
	 * @param string $campo
	 * @param bool $htmlentities
	 * @return mixed
	 */
	public function GetValue($campo, $htmlentities=true)
	{
		if(isset($this->_data[$campo]))
		{
			return $htmlentities && !in_array($campo, $this->_camposserializados) ? $this->HtmlEntities($this->_data[$campo]) : $this->_data[$campo];
		}
		else
		{
			$this->_errores[] = 'No existe el campo '.$campo;
		}

		return null;
	}
	/**
	 * Obtiene el valor del campo índice,
	 * y también sirve para establecerlo si se le pasa como parámetro, pero sin cargar los datos
	 *
	 * @param mixed $id
	 * @return mixed
	 */
	public function id($id=null)
	{
		if(!$this->_idVacio($id))
		{
			$this->_id = $id;
		}

		return $this->_id;
	}
	/**
	 * Comprueba si el valor se considera vacío para ser usado como índice
	 *
	 * @param mixed $id
	 * @return bool
	 */
	protected function _idVacio($id)
	{
		return is_null($id) || $id == "";
	}
	/**
	 * Comprueba si existe un registro con el id pasado como parámetro,
	 * o el id del objeto actual si el parámetro es null.
	 * El parámetro $id puede ser un array asociativo de campos=>valores
	 *
	 * @param mixed $id
	 * @param string $filtro
	 * @return bool
	 */
	public function Exists($id=null, $filtro='')
	{
		if(is_null($id))
		{
			$id = $this->_id;
		}

    	if(is_array($id))
    	{
    		$aux1 = array();
    		$aux2 = array();
    		foreach($id as $key=>$value)
    		{
    			$aux1[] = $value;
    			$aux2[] = $key;
    		}

    		return $this->_db->Exists($aux1, $aux2, $this->_tabla, $filtro);
    	}

		return $this->_db->Exists($id, $this->_campoid, $this->_tabla, $filtro);
	}
	/**
	 * Comprueba si existe un registro con el valor tal en el campo tal.
	 *
	 * @param string $value
	 * @param string $fieldname
	 * @param string $filtro
	 * @return bool
	 */
	public function ExistsByField($value, $fieldname, $filtro='')
	{
		return $this->_db->Exists($value, $fieldname, $this->_tabla, $filtro);
	}
	/**
	 * Comprueba si existe un registro con los valores tales en los campos tales.
	 *
	 * @param string $values
	 * @param string $fieldnames
	 * @param string $filtro
	 * @return bool
	 */
	public function ExistsByFields($values, $fieldnames, $filtro='')
	{
		return $this->_db->Exists($values, $fieldnames, $this->_tabla, $filtro);
	}
	/**
	 * Obtiene un valor por defecto genérico según el tip de campo
	 *
	 * @param type $type
	 * @return type
	 */
	protected function _type2val($type)
	{
		if(strpos($type, "int") !== false || strpos($type, "float") !== false || strpos($type, "double") !== false)
		{
			return 0;
		}
		/*
		elseif(strpos($type, "datetime"))
		{
			return "0000-00-00 00:00:00";
		}
		elseif(strpos($type, "date"))
		{
			return "0000-00-00";
		}
		*/

		return "";
	}
	/**
	 * Asigna/Obtiene el valor de un campo
	 *
	 * @param mixed $val Valor
	 * @param string $key Nombre del campo
	 * @param string $func Nombre de una función a aplicar al valor (opcional)
	 * @param mixed $minVal Valor mínimo (si se intenta asignar uno menor se pone el mínimo)
	 * @param mixed $minVal Valor máximo (si se intenta asignar uno mayor se pone el máximo)
	 * @return mixed El valor actual (tras la asignación, se haya hecho o no)
	 */
	protected function _val($val, $key, $func='', $minVal=null, $maxVal=null)
	{
		if(!is_null($val))
        {
			if($func && function_exists($func))
			{
				$val = $func($val);
			}

			if(!is_null($minVal) && $val < $minVal)
			{
				$val = $minVal;
			}

			if(!is_null($maxVal) && $val > $maxVal)
			{
				$val = $maxVal;
			}

            $this->SetValue($key, $val);
        }

        return $this->GetValue($key);
	}
	/**
	 * Activa/Desactiva un campo, es decir, le da valor 1 ó 0,
	 * intentando además actualizar el registro en la tabla.
	 * Devuelve true si puede actualizar el registro en la tabla.
	 *
	 * @param bool $valor
	 * @param string $campo
	 * @return bool
	 */
	public function OnOff($valor, $campo='activo')
	{
		if($this->SetValue($campo, $valor ? '1' : '0'))
		{
			if(!$this->_idVacio($this->_id))
			{
				if($this->_db->Execute("UPDATE `".$this->_tabla."` SET `".$campo."` = '".$this->GetValue($campo, false)."' WHERE id = ".$this->_db->quote($this->id())))
				{
					return true;
				}
				else
				{
					$this->_errores[] = $this->_db->Error();
				}
			}
			else
			{
				$this->_errores[] = 'Identificador vacío';
			}
		}

		return false;
	}
	/**
	 * Elimina el registro
	 *
	 * @param bool $resetData
	 * @return bool
	 */
	public function Delete($resetData=true)
	{
		$ok = true;

		if(!$this->_idVacio($this->_id))
		{
			if(!$this->_db->Execute(buildQuery_Delete($this->_tabla, array($this->_campoid=>$this->_id))))
			{
				$this->_errores[] = $this->_db->Error();

				$ok = false;
			}
		}

		if($resetData)
		{
			$this->ResetData();
		}

		return $ok;
	}
	/**
	 * Obtiene el texto del último error, o null si no hay errores registrados
	 *
	 * @return string
	 */
	public function Error()
	{
		$size = sizeof($this->_errores);

		return $size > 0 ? $this->_errores[$size-1] : null;
	}
	/**
	 * Aplica htmlentities con el quotestyle y charset establecidos,
	 * directamente si $var es una cadena, y recursivamente si es un array
	 *
	 * @param mixed $var
	 * @return mixed
	 */
	protected function HtmlEntities($var)
	{
		if(is_array($var))
		{
			foreach($var as $key=>$item)
			{
				$var[$key] = $this->HtmlEntities($item);
			}
		}
		else
		{
			$var = htmlentities($var, $this->_db->GetQuoteStyle(), $this->_db->GetCharSet());
		}

		return $var;
	}
}
