<?php
/**
 * Clase para manejar usuarios de SWPHP,
 * hereda de SWPHP_Reg
 *
 * Jose Carlos Cruz Parra
 * programadorphpfreelance.com
 * 2011-10-30
 *
 * SIMPLE WEB PHP EDICION ADMINISTRADA
 *
 * inc/classes/SWPHP_User.class.php
 *
 */
class SWPHP_User extends SWPHP_Reg
{
	/**
	 * Campos personalizados (array donde los índices son los camposuser_id y los elementos son los valores).
	 *
	 * @var array
	 */
	private $_camposuser;
	private $_mapacamposuser;
	/**
	 * Constructor
	 *
	 * @param DB_Connection $db
	 */
	public function __construct($id=null)
	{
        $this->_camposencriptados[] = 'pass';
		$this->_camposreadonly[] = 'buscar';

		parent::__construct(SWPHP_DBPREFIX.'users', $id);
	}
	/**
	 * Resetea los datos (campos)
	 */
	public function ResetData()
	{
		parent::ResetData();

		$this->_camposuser = array();
		$this->_mapacamposuser = array();
	}
	/**
	 * Carga un usuario
	 *
	 * @param mixed $id
	 * @return bool
	 */
	public function Load($id=null)
	{
		if(parent::Load($id))
		{
			$aux1 = $this->_db->GetValuesQuery(sprintf("SELECT c.id FROM ".SWPHP_DBPREFIX."camposuser AS c JOIN ".SWPHP_DBPREFIX."camposuser_perfiles AS cp ON cp.campouser_id = c.id WHERE cp.perfil_id = '%u' ORDER BY c.peso ASC, c.id ASC", $this->_data['perfil_id']), false);
			$aux2 = $this->_db->GetArrayForHTMLSelectQuery(sprintf("SELECT campouser_id, valor FROM ".SWPHP_DBPREFIX."users_camposuser WHERE user_id = '%u'", $this->_id), null, false);
			$this->_camposuser = array();
			foreach($aux1 as $campouser_id)
			{
				$this->_camposuser[$campouser_id] = isset($aux2[$campouser_id]) ? $aux2[$campouser_id] : '';
			}
			$this->_mapacamposuser = empty($this->_camposuser) ? array() : $this->_db->GetArrayForHTMLSelectQuery("SELECT clave, id FROM ".SWPHP_DBPREFIX."camposuser WHERE id IN(".implode(',', array_keys($this->_camposuser)).")", null, false);

			return true;
		}

		return false;
	}
	/**
	 * Guarda el usuario
	 *
	 * @return bool
	 */
	public function Save()
	{
		$this->_data['buscar'] = $this->_data['nick'].' '.$this->_data['email'];
		foreach($this->_camposuser as $campouser_id=>$valor)
		{
			$this->_data['buscar'] .= ' '.$valor;
		}

		if(parent::Save())
		{
			$this->_db->Execute(buildQuery_Delete(SWPHP_DBPREFIX."users_camposuser", array('user_id'=>$this->_id)));

			foreach($this->_camposuser as $campouser_id=>$valor)
			{
				$this->_db->Execute(buildQuery_Insert(array('user_id'=>$this->_id, 'campouser_id'=>$campouser_id, 'valor'=>$valor), SWPHP_DBPREFIX."users_camposuser"));
			}

			return true;
		}

		return false;
	}
	/**
	 * Establece los valores
	 *
	 * @param array $data
	 * @return bool
	 */
	public function SetData($data)
	{
		$ok = true;

		$camposuser = isset($data['camposuser']) && is_array($data['camposuser']) ? $data['camposuser'] : array();

		unset($data['camposuser']);

		$ok &= parent::SetData($data);

		$this->SetCamposUser($camposuser);

		return $ok;
	}
	/**
	 * Obtiene el array asociativo con los valores actualmente cargados
	 *
	 * @param bool $htmlentities
	 * @return array
	 */
	public function GetData($htmlentities=true)
	{
		$arr = parent::GetData($htmlentities);

		if($htmlentities)
		{
			foreach($this->_camposuser as $key=>$val)
			{
				$arr['camposuser'][$key] = $this->HtmlEntities($val);
			}
		}
		else
		{
			$arr['camposuser'] = $this->_camposuser;
		}

		foreach($this->_mapacamposuser as $key=>$val)
		{
			if(!isset($arr[$key]))
			{
				$arr[$key] = $arr['camposuser'][$val];
			}
		}

		return $arr;
	}
	/**
	 * Obtiene el valor de un campo (null si no lo encuentra)
	 *
	 * @param string $campo
	 * @param bool $htmlentities
	 * @return mixed
	 */
	public function GetValue($campo, $htmlentities=true)
	{
		if(isset($this->_data[$campo]))
		{
			return $htmlentities ? $this->HtmlEntities($this->_data[$campo]) : $this->_data[$campo];
		}
		elseif(isset($this->_mapacamposuser[$campo]))
		{
			return $htmlentities ? $this->HtmlEntities($this->_camposuser[$this->_mapacamposuser[$campo]]) : $this->_camposuser[$this->_mapacamposuser[$campo]];
		}

		return null;
	}
	/**
	 * Elimina el usuario
	 *
	 * @param bool $resetData
	 * @return bool
	 */
	public function Delete($resetData=true)
	{
		if($this->_db->Execute(buildQuery_Delete(SWPHP_DBPREFIX."users_camposuser", array('user_id'=>$this->_id))))
		{
			return parent::Delete($resetData);
		}

		return false;
	}

	public function SetCamposUser($camposuser=array())
	{
		$camposuserperfil = SWPHP::_GetCamposUser($this->_data['perfil_id']);
		$this->_camposuser = array();
		foreach($camposuserperfil as $campouser_id=>$campouser)
		{
			$this->_camposuser[$campouser_id] = isset($camposuser[$campouser_id]) ? $camposuser[$campouser_id] : '';
		}
	}
	/**
	 * Establece el valor para un campo personalizado
	 *
	 * @param int $campouser_id
	 * @param mixed $valor
	 * @return bool
	 */
	public function SetCampoUser($campouser_id, $valor)
	{
		if(isset($this->_camposuser[$campouser_id]))
		{
			$this->_camposuser[$campouser_id] = $valor;

			return true;
		}

		return false;
	}
	/**
	 * Obtiene el valor de un campo personalizado
	 *
	 * @param int $campouser_id
	 * @return mixed
	 */
	public function GetCampoUser($campouser_id)
	{
		if(isset($this->_camposuser[$campouser_id]))
		{
			return $this->_camposuser[$campouser_id];
		}

		return null;
	}
	/**
	 * Obtiene el conjunto de camposuser, donde los índices son los campouser_id y los valores pues los valores para esos campos
	 *
	 * @return array
	 */
	public function GetCamposUser()
	{
		return $this->_camposuser;
	}
}
