<?php
/**
 * Class to encapsulate a sql select query and manage it
 * @author José Carlos Cruz Parra AKA internia
 * josecarlos@programadorphpfreelance.com
 * http://programadorphpfreelance.com
 * 2008
 * This code is released under the GNU General Public License.
 */
//TODO:
/**
 Aunque como está ahora mismo la clase nos sirve perfectamente, se puede completar:
 - Modificadores después del contenido (por ejemplo, WITH ROLLUP para GROUP BY)
 - Otras keywords: PROCEDURE, INTO...
 - Y: FOR UPDATE, LOCK IN SHARE MODE
 - php5?

 DOCUMENTACION
 Sintaxis de select:

 SELECT
    [ALL | DISTINCT | DISTINCTROW ]
      [HIGH_PRIORITY]
      [STRAIGHT_JOIN]
      [SQL_SMALL_RESULT] [SQL_BIG_RESULT] [SQL_BUFFER_RESULT]
      [SQL_CACHE | SQL_NO_CACHE] [SQL_CALC_FOUND_ROWS]
    select_expr, ...
    [FROM table_references
    [WHERE where_condition]
    [GROUP BY {col_name | expr | position}
      [ASC | DESC], ... [WITH ROLLUP]]
    [HAVING where_condition]
    [ORDER BY {col_name | expr | position}
      [ASC | DESC], ...]
    [LIMIT {[offset,] row_count | row_count OFFSET offset}]
    [PROCEDURE procedure_name(argument_list)]
    [INTO OUTFILE 'file_name' export_options
      | INTO DUMPFILE 'file_name'
      | INTO var_name [, var_name]]
    [FOR UPDATE | LOCK IN SHARE MODE]]
*/

/**
 * Clase consulta sql select
 * Nos sirve para descomponer una consulta
 */
class SqlSelectQuery
{
	/**
	 * La consulta sql select
	 *
	 * @var string
	 */
	var $query;
	/**
	 * Array de componentes de la consulta
	 *
	 * @var array
	 */
	var $componentes;
	/**
	 * Constructor
	 *
	 * @param string $query
	 * @return SqlSelectQuery
	 */
	function SqlSelectQuery($query)
	{
		$this->query = $query;

		$this->componentes["SELECT"] = $this->getComponente("SELECT", array("ALL","DISTINCT","DISTINCTROW","HIGH_PRIORITY","STRAIGHT_JOIN","SQL_SMALL_RESULT","SQL_BIG_RESULT","SQL_BUFFER_RESULT","SQL_CACHE","SQL_NO_CACHE","SQL_CALC_FOUND_ROWS"), "FROM");
		$this->componentes["FROM"] = $this->getComponente("FROM");
		$this->componentes["WHERE"] = $this->getComponente("WHERE", array(), array("GROUP BY","HAVING","ORDER BY","LIMIT"));
		$this->componentes["GROUP BY"] = $this->getComponente("GROUP BY", array(), array("HAVING","ORDER BY","LIMIT"));
		$this->componentes["HAVING"] = $this->getComponente("HAVING", array(), array("ORDER BY","LIMIT"));
		$this->componentes["ORDER BY"] = $this->getComponente("ORDER BY", array(), array("LIMIT"));
		$this->componentes["LIMIT"] = $this->getComponente("LIMIT", array(), array(""));
	}
	/**
	 * Private
	 *
	 * @param string $keyword
	 * @param array $skip
	 * @param array $stop
	 * @return SqlSelectQueryComponente
	 */
	function getComponente($keyword, $skip=array(), $stop=array("FROM","WHERE","GROUP BY","HAVING","ORDER BY","LIMIT"))
	{
		$str = $this->query;
		$keyword = "$keyword ";
		$incremento = strlen($keyword);
		$detectar = true;
		$prevchar = false;
		if(!is_array($skip))
		{
			$skip = explode(",", $skip);
		}
		$modifiers = "";
		$contenido = "";
		for($c=0,$size=strlen($str); $c<$size; $c++)
		{
			$char = $str[$c];
			if($detectar)
			{
				if($char == "`" || $char == "(")
				{
					$detectar = false;
				}
				elseif($prevchar == " " || !$prevchar)
				{
					if(strcasecmp(substr($str, $c, $incremento), $keyword) == 0)
					{
						foreach($skip as $modifier)
						{
							$modifier = "$modifier ";
							$len = strlen($modifier);
							//echo "[".substr($str, $c+$incremento, $len)."] - [".$modifier."]<br />";
							if(strcasecmp(substr($str, $c+$incremento, $len), $modifier) == 0)
							{
								$modifiers .= $modifier;
								$c = $c + $len;
							}
						}
						$i = $c+$incremento;
						$contenido = $this->getSubstr($str, $stop, $i);
						break;
					}
				}
			}
			else
			{
				if($char == "`" || $char == ")")
				{
					$detectar = true;
				}
			}
			$prevchar = $char;
		}
		return new SqlSelectQueryComponente($keyword, $contenido, $modifiers);
	}
	/**
	 * Private
	 *
	 * @param string $str
	 * @param array $stop
	 * @param int $i
	 * @return string
	 */
	function getSubstr($str, $stop, &$i)
	{
		$detectar = true;
		$substr = "";
		if(!is_array($stop))
		{
			$stop = explode(",", $stop);
		}
		$prevchar = false;
		for($c=$i,$size=strlen($str); $c<$size; $c++)
		{
			$char = $str[$c];
			if($detectar)
			{
				if($char == "`" || $char == "(")
				{
					$substr .= $char;
					$detectar = false;
				}
				elseif($prevchar == " " || !$prevchar)
				{
					$encontrado = false;
					foreach($stop as $item)
					{
						$item = "$item ";
						$incremento = strlen($item);
						if(strcasecmp(substr($str, $c, $incremento), $item) == 0)
						{
							$encontrado = true;
							break;
						}
					}
					if($encontrado)
					{
						break;
					}
					else
					{
						$substr .= $char;
					}
				}
				else
				{
					$substr .= $char;
				}
			}
			else
			{
				$substr .= $char;
				if($char == "`" || $char == ")")
				{
					$detectar = true;
				}
			}
			$prevchar = $char;
		}
		$i = $c + $incremento;
		return trim($substr);
	}
	/**
	 * Private
	 *
	 * @param string $str
	 * @param array $stop
	 * @param int $i
	 * @return array
	 */
	function getPalabras($str, $stop, &$i)
	{
		$detectar = true;
		$palabra = "";
		$palabras = array();
		if(!is_array($stop))
		{
			$stop = explode(",", $stop);
		}
		$prevchar = false;
		for($c=$i,$size=strlen($str); $c<$size; $c++)
		{
			$char = $str[$c];
			if($detectar)
			{
				if($char == "`" || $char == "(")
				{
					$palabra .= $char;
					$detectar = false;
				}
				elseif($char == ",")
				{
					$palabras[] = trim($palabra);
					$palabra = "";
				}
				elseif($prevchar == " " || !$prevchar)
				{
					$encontrado = false;
					foreach($stop as $item)
					{
						$item = "$item ";
						$incremento = strlen($item);
						if(strcasecmp(substr($str, $c, $incremento), $item) == 0)
						{
							$palabras[] = trim($palabra);
							$palabra = "";
							$encontrado = true;
							break;
						}
					}
					if($encontrado)
					{
						break;
					}
					else
					{
						$palabra .= $char;
					}
				}
				else
				{
					$palabra .= $char;
				}
			}
			else
			{
				$palabra .= $char;
				if($char == "`" || $char == ")")
				{
					$detectar = true;
				}
			}
			$prevchar = $char;
		}
		$i = $c + $incremento;
		return $palabras;
	}
	/**
	 * Obtiene la consulta a partir de los componentes
	 *
	 * @return string
	 */
	function BuildQuery()
	{
		$query = "";
		foreach($this->componentes as $componente)
		{
			$query .= $componente->ToString()." ";
		}

		return trim($query);
	}
	/**
	 * Obtiene el contenido del componente correspondiente a $keyword
	 *
	 * @param string $keyword
	 * @return string
	 */
	function GetContenidoComponente($keyword)
	{
		return $this->componentes[strtoupper($keyword)]->GetContenido();
	}
	/**
	 * Establece el contenido para el componente correspondiente a $keyword
	 *
	 * @param string $keyword
	 * @param string $contenido
	 */
	function SetContenidoComponente($keyword, $contenido)
	{
		$this->componentes[strtoupper($keyword)]->SetContenido($contenido);
	}
	/**
	 * Establece el contenido y modificadores para el componente correspondiente a $keyword
	 *
	 * @param unknown_type $keyword
	 * @param unknown_type $contenido
	 * @param unknown_type $modifiers
	 */
	function SetComponente($keyword, $contenido, $modifiers)
	{
		$this->componentes[strtoupper($keyword)]->SetContenido($contenido);
		$this->componentes[strtoupper($keyword)]->SetModifiers($modifiers);
	}
	/**
	 * Obtiene los modificadores del componente correspondiente a $keyword
	 *
	 * @param string $keyword
	 * @return string
	 */
	function GetModifiersComponente($keyword)
	{
		return $this->componentes[strtoupper($keyword)]->GetModifiers();
	}
}
/**
 * Clase componente de una consulta sql select
 *
 */
class SqlSelectQueryComponente
{
	var $keyword;
	var $modifiers;
	var $contenido;

	function SqlSelectQueryComponente($keyword, $contenido, $modifiers="")
	{
		$this->keyword = trim($keyword);
		$this->contenido = trim($contenido);
		$this->modifiers = trim($modifiers);
	}

	function ToString()
	{
		$str = "";
		if($this->contenido)
		{
			$str = $this->keyword;
			if($this->modifiers)
			{
				$str .= " {$this->modifiers}";
			}
			$str .= " {$this->contenido}";
		}
		return $str;
	}

	function GetContenido()
	{
		return $this->contenido;
	}

	function SetContenido($contenido)
	{
		$this->contenido = $contenido;
	}

	function SetModifiers($modifiers)
	{
		$this->modifiers = $modifiers;
	}

	function GetModifiers()
	{
		return $this->modifiers;
	}
}
?>