<?php
/**
 * Jose Carlos Cruz Parra
 * programadorphpfreelance.com
 * 2012-04-08
 * para Francisco Ortega
 *
 */
class KPTL
{
	/**
	 * Obtiene un valor de configuración de KPTL
	 *
	 * @param string $variable
	 * @return mixed
	 */
	public static function GetConfigValue($variable)
	{
		return SWPHP::_GetConfigValue($variable, 'kapitaltruth', false, 'kptl_config');
	}
	/**
	 * Establece un valor de configuración de KPTL
	 *
	 * @param string $variable
	 * @param string $valor
	 * @return bool
	 */
	public static function SetConfigValue($variable, $valor)
	{
		return SWPHP::_SetConfigValue($variable, $valor, 'kapitaltruth', true, 'kptl_config');
	}

    public static function GetGruposForHTMLSelect()
    {
        global $db;

        return $db->GetArrayForHTMLSelectQuery("SELECT g.id, gi.texto FROM ".SWPHP_DBPREFIX."kptl_grupos AS g JOIN ".SWPHP_DBPREFIX."kptl_grupos_idiomas AS gi ON g.id = gi.grupo_id AND gi.idioma_id = '".SWPHP_IDIOMA_POR_DEFECTO."' ORDER BY g.peso, gi.texto");
    }
	/**
	 * Obtiene las categorías de preguntas
	 *
	 * @param bool $conPreguntas
	 * @return array
	 */
	public static function GetCategorias($conPreguntas=false, $grupos=array())
	{
		global $db;

        $filtro = "c.activo = '1' AND (c.porgrupos = '0'";
        if(!empty($grupos))
        {
            $filtro .= " OR c.grupos LIKE '%\"".implode("\"%' OR c.grupos LIKE '%\"", $grupos)."\"%'";
        }
        $filtro .= ")";

		$query = "SELECT c.id, ci.texto
			FROM ".SWPHP_DBPREFIX."kptl_categorias AS c
			LEFT JOIN ".SWPHP_DBPREFIX."kptl_categorias_idiomas AS ci ON c.id = ci.categoria_id AND ci.idioma_id = '{$_SESSION['SWPHP_IDIOMA_ACTUAL']}'
			WHERE {$filtro}
			ORDER BY c.peso ASC, c.id ASC";
		$categorias = $db->GetRows($query);

		if($conPreguntas)
		{
			array_unshift($categorias, array('id'=>0, 'texto'=>''));
		}

		foreach($categorias as $key=>$categoria)
		{
			if($categoria['id'] > 0 && is_null($categoria['texto']) || trim($categoria['texto']) == '')
			{
				$categorias[$key]['texto'] = SWPHP::_GetIdiomaValue('texto', $categoria['id'], 'categoria_id', 'kptl_categorias', SWPHP_IDIOMA_POR_DEFECTO);
			}

			if($conPreguntas)
			{
				$categorias[$key]['preguntas'] = KPTL::GetPreguntas($categoria['id'], true);
			}
		}

		return $categorias;
	}
	/**
	 * Obtiene las preguntas
	 *
	 * @param int $categoria_id
	 * @param bool $conRespuestas
	 * @return array
	 */
	public static function GetPreguntas($categoria_id=null, $conRespuestas=false)
	{
		global $db;

		$filtro = is_null($categoria_id) ? '1' : "p.categoria_id = '{$categoria_id}'";
		$query = "SELECT p.id, pi.texto, p.setrespuestas_id
			FROM ".SWPHP_DBPREFIX."kptl_preguntas AS p
			LEFT JOIN ".SWPHP_DBPREFIX."kptl_preguntas_idiomas AS pi ON p.id = pi.pregunta_id AND pi.idioma_id = '{$_SESSION['SWPHP_IDIOMA_ACTUAL']}'
			WHERE p.activo = '1' AND {$filtro}
			ORDER BY p.peso ASC, p.id ASC";

		$preguntas = $db->GetRows($query);

		foreach($preguntas as $key=>$pregunta)
		{
			if(is_null($pregunta['texto']) || trim($pregunta['texto']) == '')
			{
				$pregunta[$key]['texto'] = SWPHP::_GetIdiomaValue('texto', $pregunta['id'], 'pregunta_id', 'kptl_preguntas', SWPHP_IDIOMA_POR_DEFECTO);
			}

			if($conRespuestas)
			{
				$preguntas[$key]['respuestas'] = KPTL::GetRespuestas($pregunta['setrespuestas_id']);
			}
		}

		return $preguntas;
	}
	/**
	 * Obtiene las respuestas
	 *
	 * @param int $setrespuestas_id
	 * @return array
	 */
	public static function GetRespuestas($setrespuestas_id=null)
	{
		global $db;

		$filtro = is_null($setrespuestas_id) ? '1' : "r.setrespuestas_id = '{$setrespuestas_id}'";
		$query = "SELECT r.id, ri.texto, r.puntos, r.propia
			FROM ".SWPHP_DBPREFIX."kptl_respuestas AS r
			LEFT JOIN ".SWPHP_DBPREFIX."kptl_respuestas_idiomas AS ri ON r.id = ri.respuesta_id AND ri.idioma_id = '{$_SESSION['SWPHP_IDIOMA_ACTUAL']}'
			WHERE {$filtro}
			ORDER BY r.id ASC";

		$respuestas = $db->GetRows($query);

		foreach($respuestas as $key=>$respuesta)
		{
			if(is_null($respuesta['texto']) || trim($respuesta['texto']) == '')
			{
				$respuesta[$key]['texto'] = SWPHP::_GetIdiomaValue('texto', $respuesta['id'], 'respuesta_id', 'kptl_respuestas', SWPHP_IDIOMA_POR_DEFECTO);
			}
		}

		return $respuestas;
	}
	/**
	 * Comprueba si todas las respuestas de una valoración están pendientes de responder
	 *
	 * @param array $valoracion
	 * @return bool
	 */
	public static function ValoracionVacia($valoracion)
	{
		foreach($valoracion as $key=>$value)
		{
			if($value > -1)
			{
				return false;
			}
		}

		return true;
	}
	/**
	 * Obtiene el HTML del formulario de valoración
	 *
	 * @param mixed $user_idpasivo
	 * @param string $action
	 * @param string $onsubmit
	 * @param string $form_id
	 * @param string $popupNombreVar
	 * @return string
	 */
	public static function GetFormEscala($user_idpasivo=false, $action=null, $onsubmit='return validador.ValidarForm(this.id)', $form_id='form_escala', $popupNombreVar='popupValoracion')
	{
		global $kptl_user;

		if(is_null($action))
		{
			$action = SWPHP_PAGINA_ACTUAL;
		}

		$autovaloracion = ($user_idpasivo === false);

		if($autovaloracion)
		{
            $user_idpasivo = $kptl_user->id(); //Él mismo
			$valoracion_respuestas = $kptl_user->GetAutoValoracion();
		}
		else
		{
			$user = new KPTL_User($user_idpasivo);
			$user_autovaloracion = $user->GetAutoValoracion();

			if(KPTL::ValoracionVacia($user_autovaloracion))
			{
				return '<div class="prohibido">'.l('Este usuario aún no ha iniciado su auto-valoración, por lo tanto no puedes autorizarle verdades axiólogicas todavía.', 'kapitaltruth').'</div>';
			}

			$valoracion_respuestas = $kptl_user->GetValoracion($user_idpasivo);
		}

		$html = '<form id="'.$form_id.'" action="'.$action.'" method="post" onsubmit="'.$onsubmit.'"><div>';
		$c = 0;
		$categorias = KPTL::GetCategorias(true, $autovaloracion ? $kptl_user->GetValue('grupos', false) : $user->GetValue('grupos', false));
		foreach($categorias as $categoria)
		{
			if(sizeof($categoria['preguntas']) > 0)
			{
				$html .= "<h3>{$categoria['texto']}</h3>";
				$html .= '<table width="100%" cellpadding="0" cellspacing="0" class="listado"><tbody>';
				foreach($categoria['preguntas'] as $pregunta)
				{
                    if($autovaloracion || (isset($user_autovaloracion[$pregunta['id']]) && $user_autovaloracion[$pregunta['id']] != 0))
					{
						$c++;
						$primera = true;
						$html .= "<tr><td width=\"3%\" class=\"numeric\">{$c}.</td><td width=\"72%\">{$pregunta['texto']}</td><td width=\"25%\">";
						foreach($pregunta['respuestas'] as $respuesta)
						{
							if(!$autovaloracion || $respuesta['propia'])
							{
								$html .= sprintf('<input %s type="radio" name="preguntas[%u]" id="respuesta_%u_%u" value="%u" %s /><label for="respuesta_%u_%u"> %s</label>&nbsp;&nbsp;&nbsp;',
									$primera ? 'class="requerido" title="'.l('Pregunta', 'kapitaltruth').' '.$c.'"' : '',
									$pregunta['id'],
									$pregunta['id'],
									$respuesta['id'],
									$respuesta['id'],
									isset($valoracion_respuestas[$pregunta['id']]) && $valoracion_respuestas[$pregunta['id']] == $respuesta['id'] ? 'checked="checked"' : '',
									$pregunta['id'],
									$respuesta['id'],
									$respuesta['texto']
									);
								$primera = false;
							}
						}

						if($autovaloracion)
						{
							$html .= sprintf('<input type="radio" name="preguntas[%u]" id="respuesta_%u_%u" value="%u" %s /><label for="respuesta_%u_%u"> %s</label>',
									$pregunta['id'],
									$pregunta['id'],
									0,
									0,
									isset($valoracion_respuestas[$pregunta['id']]) && $valoracion_respuestas[$pregunta['id']] == 0 ? 'checked="checked"' : '',
									$pregunta['id'],
									0,
									l('Ignorar', 'kapitaltruth')
									);
						}

						$html .= "</td></tr>";
					}
				}
				$html .= '</tbody></table>';
			}
		}

		$html .= '<div class="center separado3">
		<input type="submit" value="'.l('Aceptar', 'kapitaltruth').'" class="boton" />
		<input type="button" value="'.l('Cancelar', 'kapitaltruth').'" class="boton" onclick="'.$popupNombreVar.'.hide();" />
	</div>
</div></form>';

		return $html;
	}
	/**
	 * Comprueba si una variable representa a un usuario de KPTL
	 *
	 * @param mixed $var
	 * @param bool $aunqueNoExista
	 * @return bool
	 */
	public static function EsKPTL_User($var, $aunqueNoExista=true)
	{
		return is_object($var) && get_class($var) == 'KPTL_User' && ($aunqueNoExista || $var->Exists());
	}
	/**
	 * Comprueba si existe un usuario de KPTL con el id indicado
	 *
	 * @param int $id
	 * @return bool
	 */
	public static function ExisteUsuario($id)
	{
		$usuario = new KPTL_User();

		return $usuario->Exists($id);
	}
	/**
	 * Obtiene el número total de usuarios de KPTL, pudiendo coger todos ($activo=null), sólo activos ($activo=true) o sólo no activos ($activo=false)
	 *
	 * @param mixed $activo
	 * @param string $filtro
	 * @return int
	 */
	public static function GetNumUsuarios($activo=null, $filtro='')
	{
		global $db;

		if(is_null($activo))
    	{
    		$aux = '';
    	}
    	elseif($activo)
    	{
    		$aux = 'WHERE u.activo = 1';
    	}
    	else
    	{
    		$aux = 'WHERE u.activo = 0';
    	}

    	return $db->GetValueQuery(KPTL_User::GetQuery(null, 'COUNT(ku.id)', mergeWhereFiltro($aux, $filtro)), false);
	}
	/**
	 * Realiza el reparto de puntos del "motorcito"
	 *
	 */
	public static function RepartirPuntos($mostrarMensajes=true)
	{
		global $db;

		//TODO: ¿log de mensajes durante el reparto?

		$habilitarkptl = KPTL::GetConfigValue('habilitarkptl');

		KPTL::SetConfigValue('habilitarkptl', false);

		$memoria = array();
		$tesoro = new KPTL_Tesoro();
		$puntosEnElTesoro = $tesoro->GetTotal();
		$users = KPTL_User::GetUsers();
		if(is_array($users) && (($size = sizeof($users)) > 0))
		{
            if($size > ini_get('max_execution_time'))
            {
                @set_time_limit($size);
            }

            if($mostrarMensajes)
            {
				SWPHP::_Msg(l('Usuarios:', 'kapitaltruth').' '.$size, 'info');
            }

			foreach($users as $key=>$user)
			{
				if($mostrarMensajes)
				{
					SWPHP::_Msg('----------------------------------------------------------', 'sep');
					SWPHP::_Msg(l('Usuario', 'kapitaltruth').' '.$user['nick'].' (Id.'.$user['id'].')', 'info');
				}

				$puntosactuales = $user['puntos_show'];

				if($mostrarMensajes)
				{
					SWPHP::_Msg($puntosactuales.' '.l('[KPTL_puntos] actuales', 'kapitaltruth'), 'info');
				}

				$puntos = 0;

		        //Votaciones
		        foreach($users as $useractivo)
		        {
		        	$puntos += $db->GetValueQuery("SELECT v.puntos FROM ".SWPHP_DBPREFIX."kptl_valoraciones AS v WHERE v.user_idactivo = '{$useractivo['id']}' AND v.user_idpasivo = '{$user['id']}' ORDER BY fecha DESC LIMIT 1", false);
		        }

		        $memoria[$user['id']]['puntuacion'] = $puntos;
		        $memoria[$user['id']]['correccion'] = 0;

		        if($mostrarMensajes)
		        {
		        	SWPHP::_Msg($puntos.' '.l('[KPTL_puntos] totalizados por valoraciones', 'kapitaltruth'), 'check');
		        }

		        //Transferencias
		        $recibidos = $user['puntos_recibidos'];
		        $emitidos = $user['puntos_emitidos'];
		        $puntos = $puntos + $recibidos - $emitidos;
		        if($mostrarMensajes)
		        {
			        SWPHP::_Msg($recibidos.' '.l('[KPTL_puntos] recibidos por transferencias', 'kapitaltruth'), 'check');
			        SWPHP::_Msg($emitidos.' '.l('[KPTL_puntos] emitidos por transferencias', 'kapitaltruth'), 'check');
			        SWPHP::_Msg($puntos.' '.l('[KPTL_puntos] de saldo tras transferencias', 'kapitaltruth'), 'check');
		        }
		        if($puntos < 0)
		        {
		        	$puntos = 0;
		        	if($mostrarMensajes)
		        	{
		        		SWPHP::_Msg(l('Este saldo se pone a cero por ser negativo', 'kapitaltruth'), 'aviso');
		        	}
		        }

		        if($puntos > 0)
		        {
		        	$puntosEnElTesoro -= $puntos;
		        }

		        $memoria[$user['id']]['fluctuacion'] = $puntos - $puntosactuales;

		        if($mostrarMensajes)
		        {
		        	SWPHP::_Msg($memoria[$user['id']]['fluctuacion'].' '.l('[KPTL_puntos] de fluctuación', 'kapitaltruth'), 'check');
		        }

		        //Actualizar los puntos del usuario
		        if($memoria[$user['id']]['fluctuacion'] != 0)
		        {
		        	$user['puntos_show'] = $puntos;

		        	$db->Execute(sprintf("UPDATE ".SWPHP_DBPREFIX."kptl_usuarios SET puntos_show = %u WHERE id = '%u'", $puntos, $user['id']));
		        }

		        $users[$key] = $user;
			}

			if($mostrarMensajes)
			{
				SWPHP::_Msg('----------------------------------------------------------', 'sep');
				SWPHP::_Msg($puntosEnElTesoro.' '.l('[KPTL_puntos] en el tesoro', 'kapitaltruth'), 'info');
			}

			//Comprobar si el tesoro ha quedado en negativo
			//(se han repartido más puntos de la cuenta)
			$umbral = 5;
			while($puntosEnElTesoro < 0)
			{
				if($mostrarMensajes)
				{
					SWPHP::_Msg(l('El tesoro ha quedado en negativo', 'kapitaltruth'), 'aviso');
					SWPHP::_Msg(l('Realizar ajuste con umbral de decimales', 'kapitaltruth').' '.$umbral, 'aviso');
				}

				//Cojo los puntos que hay de más
			    $demas = $puntosEnElTesoro * (-1);
			    $puntosRepartidos = $tesoro->GetTotal() + $demas;

			    //A cada usuario le resto el porcentaje de puntos que tiene respecto a ese total
			    //aplicado a la cantidad de puntos que hay de más
			    foreach($users as $key=>$user)
			    {
			        $puntosactuales = $user['puntos_show'];
			    	if($puntosactuales > 0)
			        {
			            $pctj = 100 / ($puntosRepartidos / $puntosactuales);
			            $restar = $demas / 100 * $pctj;
			            $parteEntera = floor($restar);
			            $parteDecimal = $restar - $parteEntera;
			            $restar = $parteDecimal >= $umbral ? $parteEntera + 1 : $parteEntera;

			            if($puntosactuales - $restar < 0)
			            {
			            	$restar = $puntosactuales;
			            }

			            if($restar > 0)
				        {
				            $user['puntos_show'] = $puntosactuales - $restar;
							$db->Execute(sprintf("UPDATE ".SWPHP_DBPREFIX."kptl_usuarios SET puntos_show = %u WHERE id = '%u'", $user['puntos_show'], $user['id']));
				            $puntosEnElTesoro += $restar;

				            $memoria[$user['id']]['correccion'] += $restar;
				            $memoria[$user['id']]['fluctuacion'] -= $restar;
			            }

			            if($mostrarMensajes)
			            {
			            	SWPHP::_Msg(l('Se restan').' '.$restar.' '.l('[KPTL_puntos] al usuario').' '.$user['nick'], 'check');
			            }
			        }

			        $users[$key] = $user;
			    }

			    if($mostrarMensajes)
			    {
			    	SWPHP::_Msg($puntosEnElTesoro.' '.l('[KPTL_puntos] en el tesoro', 'kapitaltruth'), 'info');
			    }

			    $umbral--;
			}

			//Actualizar tesoro
			$tesoro->SetActual($puntosEnElTesoro);
			$tesoro->Save();

			//Registrar puntuaciones y movimientos
			$puntuacion = new KPTL_Puntuacion();
			$movimiento = new KPTL_Movimiento();
			foreach($memoria as $user_id=>$data)
			{
				$puntuacion->Registrar($user_id, $data['puntuacion'], $data['correccion']);

				if($data['fluctuacion'])
				{
					$movimiento->Registrar($user_id, KPTL_TiposMovimiento::VALORACION, $data['fluctuacion'], l('Fluctuación por valoraciones ajenas', 'kapitaltruth'), $puntuacion->id());
				}
			}
		}
		else
		{
			if($mostrarMensajes)
			{
				SWPHP::_Msg(l('No se encontraron usuarios', 'kapitaltruth'), 'aviso');
			}
		}

		KPTL::SetConfigValue('habilitarkptl', $habilitarkptl);
	}
	/**
	 * Realiza las funciones del cron diario
	 *
	 * @param bool $forzar
	 */
	public static function CronDiario($forzar=false)
	{
		$modomotor = KPTL::GetConfigValue('modomotor');

		SWPHP::_Msg(l('Modo motor:', 'kapitaltruth').' '.$modomotor, 'info');

		if($modomotor == 'cron')
		{
			$ultima = KPTL::GetConfigValue('fechayhoraultima');
			$ahora = date('Y-m-d H:i:s');

			SWPHP::_Msg(l('Última:', 'kapitaltruth').' '.$ultima, 'info');
			SWPHP::_Msg(l('Ahora:', 'kapitaltruth').' '.$ahora, 'info');

			if($forzar)
			{
				$ejecutar = true;
				SWPHP::_Msg(l('Ejecución forzada', 'kapitaltruth'), 'aviso');
			}
			elseif(fechayhora2time($ahora) - fechayhora2time($ultima) > 86400)
			{
				$ejecutar = true;
			}
			else
			{
				$ejecutar = false;
				SWPHP::_Msg(l('Todavía no ha pasado un día completo desde la última vez que se ejecutó el cron diario.', 'kapitaltruth'), 'prohibido');
			}

			if($ejecutar)
			{
				KPTL::RepartirPuntos();
				KPTL::SetConfigValue('fechayhoraultima', $ahora);
			}
		}
		else
		{
			SWPHP::_Msg(l('La ejecución del motor de reparto no está configurada para realizarse por cron.', 'kapitaltruth'), 'prohibido');
		}

		SWPHP::_Msg(l('TERMINADO', 'kapitaltruth'), 'check');
	}
}
